c  $Id: x28f.fm4 11680 2011-03-27 17:57:51Z airwin $
c
c   Demonstrates plotting text in 3D (plmtex3, plptex3)
c
c   Copyright (C) 2007  Alan W. Irwin
c
c   This file is part of PLplot.
c
c   PLplot is free software; you can redistribute it and/or modify
c   it under the terms of the GNU Library General Public License as
c   published by the Free Software Foundation; either version 2 of the
c   License, or (at your option) any later version.
c
c   PLplot is distributed in the hope that it will be useful,
c   but WITHOUT ANY WARRANTY; without even the implied warranty of
c   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c   GNU Library General Public License for more details.
c
c   You should have received a copy of the GNU Library General Public
c   License along with PLplot; if not, write to the Free Software
c   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
c

      program x28f

      implicit none
      include 'plplot_parameters.h'

      integer XPTS, YPTS, NREVOLUTION, NROTATION, NSHEAR
      parameter(XPTS = 2)
      parameter(YPTS = 2)
      parameter(NREVOLUTION = 16)
      parameter(NROTATION = 8)
      parameter(NSHEAR = 8)
      real*8 x(XPTS), y(YPTS), z(XPTS,YPTS)
      real*8 xmin, xmax, xmid, xrange
      parameter(xmin = 0.d0)
      parameter(xmax = 1.d0)
      parameter(xmid = 0.5d0*(xmax+xmin))
      parameter(xrange = xmax-xmin)
      real*8 ymin, ymax, ymid, yrange
      parameter(ymin = 0.d0)
      parameter(ymax = 1.d0)
      parameter(ymid = 0.5d0*(ymax+ymin))
      parameter(yrange = ymax-ymin)
      real*8 zmin, zmax, zmid, zrange
      parameter(zmin = 0.d0)
      parameter(zmax = 1.d0)
      parameter(zmid = 0.5d0*(zmax+zmin))
      parameter(zrange = zmax-zmin)
      real*8 ysmin, ysmax, ysrange, dysrot, dysshear
      parameter(ysmin    = ymin + 0.1 * yrange)
      parameter(ysmax    = ymax - 0.1 * yrange)
      parameter(ysrange  = ysmax - ysmin)
      parameter(dysrot   = ysrange / ( NROTATION - 1 ))
      parameter(dysshear = ysrange / ( NSHEAR - 1 ))
      real*8 zsmin, zsmax, zsrange, dzsrot, dzsshear
      parameter(zsmin    = zmin + 0.1 * zrange)
      parameter(zsmax    = zmax - 0.1 * zrange)
      parameter(zsrange  = zsmax - zsmin)
      parameter(dzsrot   = zsrange / ( NROTATION - 1 ))
      parameter(dzsshear = zsrange / ( NSHEAR - 1 ))
      real*8 ys, zs
      real*8 x_inclination, y_inclination, z_inclination,
     &  x_shear, y_shear, z_shear,
     &  omega, sin_omega, cos_omega, domega
      integer i, j
      real*8 radius, pitch, xpos, ypos, zpos
      integer lnblnk

      character*80 pstring
      pstring =
     &  "The future of our civilization depends on software freedom."

      do i = 1,XPTS
      	 x(i) = xmin + dble(i-1)*(xmax-xmin)/dble(XPTS-1)
      enddo

      do j = 1,YPTS
      	 y(j) = ymin + dble(j-1)*(ymax-ymin)/dble(YPTS-1)
      enddo
       
      do j = 1,YPTS
        do i = 1,XPTS
          z(i,j) = 0.d0
        enddo
      enddo

c      Parse and process command line arguments

      call plparseopts(PL_PARSE_FULL)

      call plinit()

c      Page 1: Demonstrate inclination and shear capability pattern.

      call pladv(0)
      call plvpor(-0.15d0, 1.15d0, -0.05d0, 1.05d0)
      call plwind(-1.2d0, 1.2d0, -0.8d0, 1.5d0)
      call plw3d(1.0d0, 1.0d0, 1.0d0,
     &  xmin, xmax, ymin, ymax, zmin, zmax,
     &  20.d0, 45.d0)

      call plcol0(2)
      call plbox3("b",   "", xmax-xmin, 0,
     & "b",   "", ymax-ymin, 0,
     & "bcd", "", zmax-zmin, 0)

c      z = zmin.
      call plschr(0.d0, 1.0d0)
      do i = 1,NREVOLUTION
         omega = 2.d0*PI*(dble(i-1)/dble(NREVOLUTION))
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         x_inclination = 0.5d0*xrange*cos_omega
         y_inclination = 0.5d0*yrange*sin_omega
         z_inclination = 0.d0
         x_shear = -0.5d0*xrange*sin_omega
         y_shear = 0.5d0*yrange*cos_omega
         z_shear = 0.d0
         call plptex3(
     &     xmid, ymid, zmin,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.0d0, "  revolution")
      enddo

c      x = xmax.
      call plschr(0.d0, 1.0d0)
      do i = 1,NREVOLUTION
         omega = 2.d0*PI*(dble(i-1)/dble(NREVOLUTION))
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         x_inclination = 0.d0
         y_inclination = -0.5d0*yrange*cos_omega
         z_inclination = 0.5d0*zrange*sin_omega
         x_shear = 0.d0
         y_shear = 0.5d0*yrange*sin_omega
         z_shear = 0.5d0*zrange*cos_omega
         call plptex3(
     &     xmax, ymid, zmid,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.0d0, "  revolution")
      enddo

c      y = ymax.
      call plschr(0.d0, 1.0d0)
      do i = 1,NREVOLUTION
         omega = 2.d0*PI*(dble(i-1)/dble(NREVOLUTION))
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         x_inclination = 0.5d0*xrange*cos_omega
         y_inclination = 0.d0
         z_inclination = 0.5d0*zrange*sin_omega
         x_shear = -0.5d0*xrange*sin_omega
         y_shear = 0.d0
         z_shear = 0.5d0*zrange*cos_omega
         call plptex3(
     &     xmid, ymax, zmid,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.0d0, "  revolution")
      enddo
c      Draw minimal 3D grid to finish defining the 3D box.
      call plmesh(x, y, z, XPTS, YPTS, DRAW_LINEXY, XPTS)

c      Page 2: Demonstrate rotation of string around its axis.
      call pladv(0)
      call plvpor(-0.15d0, 1.15d0, -0.05d0, 1.05d0)
      call plwind(-1.2d0, 1.2d0, -0.8d0, 1.5d0)
      call plw3d(1.0d0, 1.0d0, 1.0d0,
     &  xmin, xmax, ymin, ymax, zmin, zmax,
     &  20.d0, 45.d0)

      call plcol0(2)
      call plbox3("b",   "", xmax-xmin, 0,
     & "b",   "", ymax-ymin, 0,
     & "bcd", "", zmax-zmin, 0)

c      y = ymax.
      call plschr(0.d0, 1.0d0)
      x_inclination = 1.d0
      y_inclination = 0.d0
      z_inclination = 0.d0
      x_shear = 0.d0
      do i = 1,NROTATION
         omega = 2.d0*PI*dble(i-1)/dble(NROTATION)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         y_shear = 0.5d0*yrange*sin_omega
         z_shear = 0.5d0*zrange*cos_omega
         zs        = zsmax - dzsrot * dble(i-1)
         call plptex3(
     &     xmid, ymax, zs,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "rotation for y = y#dmax#u")
      enddo

c      x = xmax.
      call plschr(0.d0, 1.0d0)
      x_inclination = 0.d0
      y_inclination = -1.d0
      z_inclination = 0.d0
      y_shear = 0.d0
      do i = 1,NROTATION
         omega = 2.d0*PI*dble(i-1)/dble(NROTATION)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         x_shear = 0.5d0*xrange*sin_omega
         z_shear = 0.5d0*zrange*cos_omega
         zs        = zsmax - dzsrot * dble(i-1)
         call plptex3(
     &     xmax, ymid, zs,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "rotation for x = x#dmax#u")
      enddo

c      z = zmin.
      call plschr(0.d0, 1.0d0)
      x_inclination = 1.d0
      y_inclination = 0.d0
      z_inclination = 0.d0
      x_shear = 0.d0
      do i = 1,NROTATION
         omega = 2.d0*PI*dble(i-1)/dble(NROTATION)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         y_shear = 0.5d0*yrange*cos_omega
         z_shear = 0.5d0*zrange*sin_omega
         ys        = ysmax - dysrot * dble(i-1)
         call plptex3(
     &     xmid, ys, zmin,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "rotation for z = z#dmin#u")
      enddo
c      Draw minimal 3D grid to finish defining the 3D box.
      call plmesh(x, y, z, XPTS, YPTS, DRAW_LINEXY, XPTS)

c      Page 3: Demonstrate shear of string along its axis.
c      Work around xcairo and pngcairo (but not pscairo) problems for
c      shear vector too close to axis of string. (N.B. no workaround
c      would be domega = 0.d0.)
      domega = 0.05d0
      call pladv(0)
      call plvpor(-0.15d0, 1.15d0, -0.05d0, 1.05d0)
      call plwind(-1.2d0, 1.2d0, -0.8d0, 1.5d0)
      call plw3d(1.0d0, 1.0d0, 1.0d0,
     &  xmin, xmax, ymin, ymax, zmin, zmax,
     &  20.d0, 45.d0)

      call plcol0(2)
      call plbox3("b",   "", xmax-xmin, 0,
     & "b",   "", ymax-ymin, 0,
     & "bcd", "", zmax-zmin, 0)

c      y = ymax.
      call plschr(0.d0, 1.0d0)
      x_inclination = 1.d0
      y_inclination = 0.d0
      z_inclination = 0.d0
      y_shear = 0.d0
      do i = 1,NSHEAR
         omega = domega + 2.d0*PI*dble(i-1)/dble(NSHEAR)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         x_shear = 0.5d0*xrange*sin_omega
         z_shear = 0.5d0*zrange*cos_omega
         zs        = zsmax - dzsshear * dble(i-1)
         call plptex3(
     &     xmid, ymax, zs,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "shear for y = y#dmax#u")
      enddo

c      x = xmax.
      call plschr(0.d0, 1.0d0)
      x_inclination = 0.d0
      y_inclination = -1.d0
      z_inclination = 0.d0
      x_shear = 0.d0
      do i = 1,NSHEAR
         omega = domega + 2.d0*PI*dble(i-1)/dble(NSHEAR)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         y_shear = -0.5d0*yrange*sin_omega
         z_shear = 0.5d0*zrange*cos_omega
         zs        = zsmax - dzsshear * dble(i-1)
         call plptex3(
     &     xmax, ymid, zs,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "shear for x = x#dmax#u")
      enddo

c      z = zmin.
      call plschr(0.d0, 1.0d0)
      x_inclination = 1.d0
      y_inclination = 0.d0
      z_inclination = 0.d0
      z_shear = 0.d0
      do i = 1,NSHEAR
         omega = domega + 2.d0*PI*dble(i-1)/dble(NSHEAR)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
         y_shear = 0.5d0*yrange*cos_omega
         x_shear = 0.5d0*xrange*sin_omega
         ys        = ysmax - dysshear * dble(i-1)
         call plptex3(
     &     xmid, ys, zmin,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, "shear for z = z#dmin#u")
      enddo

c      Draw minimal 3D grid to finish defining the 3D box.
      call plmesh(x, y, z, XPTS, YPTS, DRAW_LINEXY, XPTS)

c      Page 4: Demonstrate drawing a string on a 3D path.
      call pladv(0)
      call plvpor(-0.15d0, 1.15d0, -0.05d0, 1.05d0)
      call plwind(-1.2d0, 1.2d0, -0.8d0, 1.5d0)
      call plw3d(1.0d0, 1.0d0, 1.0d0,
     &  xmin, xmax, ymin, ymax, zmin, zmax,
     &  40.d0, -30.d0)

      call plcol0(2)
      call plbox3("b",   "", xmax-xmin, 0,
     & "b",   "", ymax-ymin, 0,
     & "bcd", "", zmax-zmin, 0)

      call plschr(0.d0, 1.2d0)
c      domega controls the spacing between the various characters of the
c      string and also the maximum value of omega for the given number
c      of characters in pstring.
      domega = 2.d0*PI/lnblnk(pstring)
      omega = 0.d0
c      3D function is a helix of the given radius and pitch
      radius = 0.5d0
      pitch = 1.d0/(2.d0*PI)
      do i = 1,lnblnk(pstring)
      	 sin_omega = sin(omega)
      	 cos_omega = cos(omega)
      	 xpos = xmid + radius*sin_omega
      	 ypos = ymid - radius*cos_omega
      	 zpos = zmin + pitch*omega
c         In general, the inclination is proportional to the derivative of
c         the position wrt theta.
      	 x_inclination = radius*cos_omega
      	 y_inclination = radius*sin_omega
      	 z_inclination = pitch
c         The shear vector should be perpendicular to the 3D line with Z
c 	  component maximized, but for low pitch a good approximation is
c 	  a constant vector that is parallel to the Z axis.
         x_shear = 0.d0
         y_shear = 0.d0
         z_shear = 1.d0
         call plptex3(
     &     xpos, ypos, zpos,
     &     x_inclination, y_inclination, z_inclination,
     &     x_shear, y_shear, z_shear,
     &     0.5d0, pstring(i:i))
      	   omega = omega + domega
      enddo
c      Draw minimal 3D grid to finish defining the 3D box.
      call plmesh(x, y, z, XPTS, YPTS, DRAW_LINEXY, XPTS)

c Page 5: Demonstrate plmtex3 axis labelling capability
      call pladv(0)
      call plvpor(-0.15d0, 1.15d0, -0.05d0, 1.05d0)
      call plwind(-1.2d0, 1.2d0, -0.8d0, 1.5d0)
      call plw3d(1.0d0, 1.0d0, 1.0d0,
     &  xmin, xmax, ymin, ymax, zmin, zmax,
     &  20.d0, 45.d0)

      call plcol0(2)
      call plbox3("b",   "", xmax-xmin, 0,
     & "b",   "", ymax-ymin, 0,
     & "bcd", "", zmax-zmin, 0)

      call plschr(0.d0, 1.0d0)
      call plmtex3("xp", 3.0d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("xp", 4.5d0, 0.5d0, 0.5d0, "primary X-axis label")
      call plmtex3("xs", -2.5d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("xs", -1.0d0, 0.5d0, 0.5d0, "secondary X-axis label")
      call plmtex3("yp", 3.0d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("yp", 4.5d0, 0.5d0, 0.5d0, "primary Y-axis label")
      call plmtex3("ys", -2.5d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("ys", -1.0d0, 0.5d0, 0.5d0, "secondary Y-axis label")
      call plmtex3("zp", 4.5d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("zp", 3.0d0, 0.5d0, 0.5d0, "primary Z-axis label")
      call plmtex3("zs", -2.5d0, 0.5d0, 0.5d0, "Arbitrarily displaced")
      call plmtex3("zs", -1.0d0, 0.5d0, 0.5d0, "secondary Z-axis label")
c      Draw minimal 3D grid to finish defining the 3D box.
      call plmesh(x, y, z, XPTS, YPTS, DRAW_LINEXY, XPTS)

      call plend()
      stop
      end
